//+------------------------------------------------------------------+
//|                                                      MARSICD.mq4 |
//|                                                  John Q. Aimsson |
//|                                                aimsson@gmail.com |
//+------------------------------------------------------------------+

#property copyright "John Q. Aimsson"
#property link      "aimsson@gmail.com"
#property indicator_separate_window
#property indicator_minimum -70
#property indicator_maximum 70
#property indicator_level1 -30
#property indicator_level2 -70
#property indicator_level3 -50
#property indicator_level4  30
#property indicator_level5  70
#property indicator_level6  50
#property indicator_buffers 7
#property indicator_color3 Green
#property indicator_color4 Maroon
#property indicator_color1 LimeGreen
#property indicator_color2 PaleVioletRed
#property indicator_color5 MediumSeaGreen
#property indicator_color6 Red
#property indicator_color7 Salmon
#property indicator_width5 2
#property indicator_width6 2
#property indicator_style7 STYLE_DOT
//---- input parameters
extern string TimeFrame = "Current time frame";
extern int ARSI_PERIOD=14;
extern int RRSI_PERIOD=14;
extern int AMA_PERIOD=5;
extern int RMA_PERIOD=5;
extern int sMAD_PERIOD=21;
extern bool Interpolate = true;
bool ShiftDown = true;
//---- buffers
double ARSI_BUF[];
double RRSI_BUF[];
double AMA_BUF[];
double RMA_BUF[];
double DeltaUp[];
double DeltaDn[];
double Delta[];
double sMAD[];

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(8); 
   SetIndexBuffer(0,DeltaUp);     SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,DeltaDn);     SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,ARSI_BUF);
   SetIndexBuffer(3,RRSI_BUF);
   SetIndexBuffer(4,AMA_BUF);
   SetIndexBuffer(5,RMA_BUF);
   SetIndexBuffer(6,sMAD);
   SetIndexBuffer(7,Delta);
   
   SetIndexDrawBegin(0,ARSI_PERIOD);
   SetIndexDrawBegin(1,RRSI_PERIOD);
   SetIndexDrawBegin(3,ARSI_PERIOD+AMA_PERIOD);
   SetIndexDrawBegin(4,RRSI_PERIOD+RMA_PERIOD);
   SetIndexDrawBegin(5,sMAD_PERIOD);
   
         //
         //
         //
         //
         //
                  
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
         //
         //
         //
         //
         //

   IndicatorShortName(timeFrameToString(timeFrame)+" MARSICD ("+ARSI_PERIOD+","+RRSI_PERIOD+","+AMA_PERIOD+","+RMA_PERIOD+","+sMAD_PERIOD+")");
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Relative Strength Index                                          |
//+------------------------------------------------------------------+
int start()
  {
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { DeltaUp[0] = limit+1; return(0); }
  
   if (calculateValue || timeFrame==Period())
   {
      for(int i=limit; i>=0; i--) 
        {
         ARSI_BUF[i]=iRSI(NULL,0,ARSI_PERIOD,6,i);
         RRSI_BUF[i]=100.0-iRSI(NULL,0,RRSI_PERIOD,6,i);
         if (ShiftDown)
         {
            ARSI_BUF[i] -= 50;
            RRSI_BUF[i] -= 50;
         }
      }
      for(i=limit; i>=0; i--) 
      {
         AMA_BUF[i]=iMAOnArray(ARSI_BUF,Bars,AMA_PERIOD,0,MODE_LWMA,i);
         RMA_BUF[i]=iMAOnArray(RRSI_BUF,Bars,RMA_PERIOD,0,MODE_LWMA,i);
         Delta[i]=AMA_BUF[i]-RMA_BUF[i];
         DeltaUp[i] = EMPTY_VALUE;
         DeltaDn[i] = EMPTY_VALUE;
            if (Delta[i]>0) DeltaUp[i] = Delta[i];
            if (Delta[i]<0) DeltaDn[i] = Delta[i];
      }
      for(i=limit; i>=0; i--) sMAD[i]=iMAOnArray(Delta,Bars,sMAD_PERIOD,0,MODE_LWMA,i);
      return(0);
   }

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         ARSI_BUF[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,2,y);
         RRSI_BUF[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,3,y);
         DeltaUp[i]  = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,0,y);
         DeltaDn[i]  = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,1,y);
         AMA_BUF[i]  = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,4,y);
         RMA_BUF[i]  = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,5,y);
         sMAD[i]     = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,6,y);
         Delta[i]    = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",ARSI_PERIOD,RRSI_PERIOD,AMA_PERIOD,RMA_PERIOD,sMAD_PERIOD,7,y);

         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               ARSI_BUF[i+k] = ARSI_BUF[i] + (ARSI_BUF[i+n]-ARSI_BUF[i])*k/n;
               RRSI_BUF[i+k] = RRSI_BUF[i] + (RRSI_BUF[i+n]-RRSI_BUF[i])*k/n;
               if (DeltaUp[i] !=EMPTY_VALUE) DeltaUp[i+k] = Delta[i] + (Delta[i+n]-Delta[i])*k/n;
               if (DeltaDn[i] !=EMPTY_VALUE) DeltaDn[i+k] = Delta[i] + (Delta[i+n]-Delta[i])*k/n;
               AMA_BUF[i+k]  = AMA_BUF[i]  + (AMA_BUF[i+n] -AMA_BUF[i] )*k/n;
               RMA_BUF[i+k]  = RMA_BUF[i]  + (RMA_BUF[i+n] -RMA_BUF[i] )*k/n;
               sMAD[i+k]     = sMAD[i]     + (sMAD[i+n]    -sMAD[i]    )*k/n;
            }               
   }
   return(0);   
}   

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char1 = StringGetChar(s, length);
         if((char1 > 96 && char1 < 123) || (char1 > 223 && char1 < 256))
                     s = StringSetChar(s, length, char1 - 32);
         else if(char1 > -33 && char1 < 0)
                     s = StringSetChar(s, length, char1 + 224);
   }
   return(s);
}